#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/20 22:47
@File  : __init__.py
'''
'''
PO模式：
    1、基础元素定位函数的封装（基础定位、键盘、鼠标事件，解决：定位不稳定问题）
    2、页面元素的定位（定位层）
    3、页面元素的操作（操作层）
    4、界面业务的操作（业务层）
    5、基于单元模块的用例（用例层）
'''
